@@ -5,4 +5,6 @@ Python HKP client module
 from hkp.client import Key, Identity, KeyServer
 
 
-__all__ = ['Key', 'Identity', 'KeyServer']
+VERSION = (0, 1, 3)
+
+__all__ = ['Key', 'Identity', 'KeyServer', 'VERSION']
