@@ -9,14 +9,14 @@ import unittest
 from random import choice
 from hkp import KeyServer, Identity, Key
 from hkp.client import ALGORITHMS
+from dingus import patch, Dingus
+from urllib import urlencode
 
 
 __all__ = ['TestKeyServer', 'TestIdentity', 'TestKey']
 
-KEY_SERVERS = (
-    'http://pool.sks-keyservers.net',
-    'http://pgp.mit.edu',
-)
+#Tests are isolated from network, so keyserver is not really hit.
+KEY_SERVER = 'http://pool.sks-keyservers.net'
 
 PORT = 11371
 
@@ -24,6 +24,20 @@ PORT = 11371
 FINGERPRINT = '0x46181433FBB75451'
 KEYID = '0x%s' % FINGERPRINT[-8:]
 UID = 'Ubuntu CD Image Automatic Signing Key'
+FULL_UID = UID + ' <cdimage@ubuntu.com>'
+DINGUS = Dingus()
+
+
+def load_fixture(name):
+    """
+    Return a file-like fixture, just like urlopen would.
+    """
+    return open(os.path.join(
+            os.path.abspath(os.path.dirname(__file__)),
+            name,
+            ),
+        'r'
+        )
 
 
 class TestKeyServer(unittest.TestCase):
@@ -35,10 +49,12 @@ class TestKeyServer(unittest.TestCase):
         """
         Set up random KeyServer.
         """
-        self.server_host = choice(KEY_SERVERS)
+        self.server_host = KEY_SERVER
         self.serv = KeyServer(self.server_host)
         self.begin_header = '-----BEGIN PGP PUBLIC KEY BLOCK-----'
         self.end_header = '-----END PGP PUBLIC KEY BLOCK-----'
+        DINGUS.reset()
+        DINGUS.return_value = load_fixture('search_answer')
 
     def test_init(self):
         """
@@ -47,39 +63,65 @@ class TestKeyServer(unittest.TestCase):
         self.assertEqual(self.serv.host, self.server_host)
         self.assertEqual(self.serv.port, PORT)
 
-    def test_search(self):
+    @patch('urllib2.urlopen', DINGUS)
+    def test_search_by_id(self):
         """
-        Test search with keyid, fingerprint and uid.
+        Test search with keyid.
         """
         result = self.serv.search(KEYID)
+        search_url = (self.server_host + ':11371/pks/lookup'
+                '?search=' + KEYID + '&exact=off&options=mr&op=index')
+        self.assertTrue(DINGUS.calls('()', search_url).once())
         self.assertEqual(len(result), 1)
-        result[0].keyid = KEYID
-        result[0].identities[0].uid = UID
+        self.assertEqual(result[0].keyid, KEYID[2:])
+        self.assertEqual(result[0].identities[0].uid, FULL_UID)
 
+    @patch('urllib2.urlopen', DINGUS)
+    def test_search_by_fingerprint(self):
+        """
+        Test search with fingerprint.
+        """
         result = self.serv.search(FINGERPRINT)
+        search_url = (self.server_host + ':11371/pks/lookup'
+                '?search=' + FINGERPRINT + '&exact=off&options=mr&op=index')
+        self.assertTrue(DINGUS.calls('()', search_url).once())
         self.assertEqual(len(result), 1)
-        result[0].keyid = KEYID
-        result[0].identities[0].uid = UID
+        self.assertEqual(result[0].keyid, KEYID[2:])
+        self.assertEqual(result[0].identities[0].uid, FULL_UID)
 
+    @patch('urllib2.urlopen', DINGUS)
+    def test_search_by_uid(self):
+        """
+        Test search with uid.
+        """
         result = self.serv.search(UID)
+        search_url = (self.server_host + ':11371/pks/lookup'
+                '?search=' + UID.replace(' ', '+') +
+                '&exact=off&options=mr&op=index')
+        self.assertTrue(DINGUS.calls('()', search_url).once())
         self.assertEqual(len(result), 1)
-        result[0].keyid = KEYID
-        result[0].identities[0].uid = UID
+        self.assertEqual(result[0].keyid, KEYID[2:])
+        self.assertEqual(result[0].identities[0].uid, FULL_UID)
 
+    @patch('urllib2.urlopen', DINGUS)
     def test_add(self):
         """
         Test ASCII armored key upload
         """
-        stored_key = os.path.join(
-            os.path.abspath(os.path.dirname(__file__)),
-            'ubuntu.key',
-        )
-        key = open(stored_key, 'r').read()
-        self.serv.add('%s\n\n%s\n%s' % (
+        stored_key = 'ubuntu.key'
+        key = load_fixture(stored_key).read()
+        keytext = '%s\n\n%s\n%s' % (
             self.begin_header,
             key,
             self.end_header,
-        ))
+        )
+
+        self.serv.add(keytext)
+
+        add_url = self.server_host + ':11371/pks/add'
+        self.assertTrue(DINGUS.calls('()', add_url,
+            urlencode({'keytext': keytext})
+            ).once())
 
 
 class TestIdentity(unittest.TestCase):
@@ -132,18 +174,17 @@ class TestKey(unittest.TestCase):
         """
         Set up Key data.
         """
-        self.server_host = choice(KEY_SERVERS)
+        self.server_host = KEY_SERVER
         self.algo = str(choice(ALGORITHMS.keys()))
         self.key_length = '2048'
         self.creation_date = time.time()
         self.expiration_date = time.time()
         self.flags = 'dr'
-        self.stored_key = os.path.join(
-            os.path.abspath(os.path.dirname(__file__)),
-            'ubuntu.key',
-        )
+        self.stored_key = 'ubuntu.key'
         self.begin_header = '-----BEGIN PGP PUBLIC KEY BLOCK-----'
         self.end_header = '-----END PGP PUBLIC KEY BLOCK-----'
+        self.key_answer = load_fixture('ubuntu.html')
+        DINGUS.reset()
 
     def test_init(self):
         """
@@ -182,6 +223,7 @@ class TestKey(unittest.TestCase):
             expiration_date.replace(microsecond=0),
         )
 
+    @patch('urllib2.urlopen', DINGUS)
     def test_key(self):
         """
         Retrieve Ubuntu ASCII armored public key and check it.
@@ -196,8 +238,8 @@ class TestKey(unittest.TestCase):
             self.expiration_date,
             self.flags,
         )
-
-        stored_key = open(self.stored_key, 'r').read()
+        DINGUS.return_value = self.key_answer
+        stored_key = load_fixture(self.stored_key).read()
 
         # key property is absent before first call
         self.assertFalse('key' in key.__dict__)
